# OB-USP-Agent
OBUSPA is fork of [BBFs obuspa](https://dev.iopsys.eu/fork/obuspa/-/tree/devel/README.md) project with vendor integration of [uspd](https://dev.iopsys.eu/iopsys/uspd).

## Project Components

Project consists of following components:
- Application itself written in C programming language
- Documentation in a Markdown format
- Module tests usage bash scripts and python

## Build Instructions

obsupa is written using C programming language and depends on a number of components found in `OpenWrt` for building and running. It is advised to use iopsys feeds to resolve the dependencies.

## UCI Config

OBUSPA requires a configuration file to provide input parameters. It basically has uci options defined in global section which either configures the command line options for OBUSPA or non-datamodel inputs, rest of sections are generally a way to define parameters for below datamodel objects
```
Device.LocalAgent.
Device.MQTT.Client.
Device.STOMP.Connection.
```
Below is an example uci, complete list of supported uci parameters is available [link](./docs/api/uci/obuspa.md), which is generated from [uci schema](https://dev.iopsys.eu/fork/obuspa/-/tree/devel/schemas/uci/obuspa.json)
```shellscript
config obuspa 'global'
	option enabled '1'
	option interface 'wan'
	#option ifname 'eth0.1'
	option debug '0'
	option log_level '1'
	#option db_file '/etc/obuspa/usp.db'
	#option role_file '/etc/obuspa/roles.json'
	option dm_caching_exclude '/etc/obuspa/dmcaching_exclude.json'
	#option trust_cert '/etc/obuspa/rootCA.pem'
	#option client_cert '/etc/obuspa/client.pem'
	#option log_dest '/var/log/obuspa'

config localagent 'localagent'
	option Enable '1'

config controller 'localcontroller'
	option EndpointID 'proto::interop-usp-controller'
	option Protocol 'MQTT'
	option Topic '/usp/controller'
	option mqtt 'localmqtt'
	option assigned_role_name 'full_access'

config mtp
	option Protocol 'MQTT'
	option ResponseTopicConfigured '/usp/endpoint'
	option mqtt 'localmqtt'

config mqtt 'localmqtt'
	option BrokerAddress '127.0.0.1'
	option BrokerPort '1883'
	option TransportProtocol 'TCP/IP'
```

## Important Topics
* [Concepts and Workflow](./docs/guide/workflow.md)
* [Integration Notes](./docs/guide/obuspa-3.0_integration_notes_and_observation.md)
* [How to add controller trust roles](./docs/guide/controller_trust.md)
* [How to override the default role definitions](./docs/guide/override_role.md)
* [How to change the max caching interval for multi-instance objects](./docs/guide/caching.md#how-to-change-the-max-caching-interval-for-multi-instance-objects)
* [How to optimize the group calls](./docs/guide/optimize_group_call.md)
* [How to configure the ubus timeouts](./docs/guide/ubus_timeout.md)
* [How does obuspa registers for new schema](./docs/guide/register_schema.md)
* [How to define a multi-instance object in no caching list](./docs/guide/caching.md#how-to-define-a-multi-instance-object-in-no-caching-list)

## Dependencies
OBUSPA has some compile time dependency and some run time dependencies as listed below:

### Build-Time Dependencies

To successfully build OBUSPA, following libraries are needed:

| Dependency   |                    Link                     |    License        |
| ------------ | ------------------------------------------- | ----------------- |
| libuci       | https://git.openwrt.org/project/uci.git     | LGPL 2.1          |
| libubox      | https://git.openwrt.org/project/libubox.git | BSD               |
| libubus      | https://git.openwrt.org/project/ubus.git    | LGPL 2.1          |
| libopenssl   | http://www.openssl.org                      | Apache License 2.0|
| libcurl      | https://github.com/curl/curl.git            | MIT               |
| libsqlite3   | https://www.sqlite.org/2020/                | PUBLICDOMAIN      |
| libmosquitto | https://mosquitto.org/files/source/         | EPL-2.0           |
| uspd         | https://dev.iopsys.eu/iopsys/uspd.git       | LGPL 2.1          |


### Run-Time Dependencies

In order to run the OBUSPA, following dependencies are needed to be running/available before OBUSPA.

| Dependency |                   Link                   | License  |
| ---------- | ---------------------------------------- | -------- |
| ubusd      | https://git.openwrt.org/project/ubus.git | LGPL 2.1 |
| uspd       | https://dev.iopsys.eu/iopsys/uspd.git    | LGPL 2.1 |

`uspd` is used to expose the datamodel object over `ubus`, it is the datastore for all datamodel objects except Device.LocalAgent.
